import { IAdapter } from '../adapters';
import { CalleeAgent } from '../agents';
import { Broker, BrokerMessage, BrokerRequest } from './broker';
/**
 * Called by `CallerBroker`
 */
export declare class CalleeBroker extends Broker {
    protected readonly handlers: Map<string, CalleeBroker.MessageRawHandler>;
    protected readonly agents: Map<string, CalleeAgent>;
    constructor(options: CalleeBroker.Options);
    protected reply(id: string, error?: Error, payload?: any): void;
    protected handle(msg: BrokerRequest | BrokerMessage): void;
    protected onAdapterMessage(msg: unknown): void;
    /**
     * Publish an event
     * @param msg Message to send
     */
    publish(msg: BrokerMessage): void;
    /**
     * Set a raw handler for the given message type
     * @param type Message type
     * @param handler Message handler
     * @returns Remove handler function
     * @description This will override previously set handler of the same type
     */
    setRawHandler(type: string, handler: CalleeBroker.MessageRawHandler): () => void;
    /**
     * Set a wrapped handler for the given message type
     * @param type Message type
     * @param handler Message handler
     * @returns Remove handler function
     * @description
     * - This will override previously set handler of the same type
     * - support async
     * - support return value as reply data
     * - errors will be caught and sent
     */
    setHandler(type: string, handler: CalleeBroker.MessageHandler): () => void;
    /**
     * Remove handler of the given type
     * @param type Message type
     */
    removeHandler(type: string): void;
    /**
     * Inject agent
     * @param key Agent key
     * @param target Agent target
     * @returns Eject agent funtion
     */
    injectAgent(key: string, target: any): () => void;
    /**
     * Eject agent
     * @param key Agent key
     * @returns `true` if an agent with the given key has been ejected
     */
    ejectAgent(key: string): boolean;
    /**
     * Reset data
     */
    reset(): void;
}
export declare namespace CalleeBroker {
    interface Options {
        /**
         * Adapter
         */
        adapter: IAdapter;
    }
    interface RequestContext {
        /**
         * check if the current request is done
         */
        readonly isDone: boolean;
        /**
         * Finish current request without response data
         */
        end(): void;
        /**
         * Finish current request with error
         * @param error Error
         */
        end(error: Error): void;
        /**
         * Finish current request with error or data
         * @param error Error
         * @param data Data
         */
        end(error: undefined, data: any): void;
    }
    interface MessageHandlerContext {
        /**
         * Request context
         */
        readonly req?: RequestContext;
    }
    type MessageRawHandler = (ctx: MessageHandlerContext, payload?: any) => void;
    type MessageHandler = (ctx: MessageHandlerContext, payload?: any) => any;
    type Plugin = (broker: CalleeBroker) => void;
}
